'''grade
'''

from performance import Performance
import rmstats
from copy import deepcopy
from performance import _split


class GradeAnalysis(Performance):
    '''Analysis of grade calculation in superclass Performance.
    This is an example class. Make your own by subclassing
    GradeAnalysis or Performance as, or if, appropriate.
    '''
    

    def __init__(self, folder=None):

        super(GradeAnalysis, self).__init__(folder)

        self.statistics = {}
        

    def GetGradeStatistics(self, calculate=False):

        s = self.statistics
        if calculate:
            s.clear()

        if not len(self.statistics):
            s['MaxGrade'] = max(self.grades.values())
            s['MaxInitialGrade'] = rmstats.MaxDictList(self.performances, 0)
            s['MeanDiffGapScorePrediction'] = rmstats.MeanDiffListSparse(self.gapscores,
                                                                         self.gappredictedscores,
                                                                         self.gapgames)
            s['MeanDiffGapScorePredictionHigh'] = rmstats.MeanDiffListSparse(self.gapscores[-_split:],
                                                                             self.gappredictedscores[-_split:],
                                                                             self.gapgames[-_split:])
            s['MeanDiffGapScorePredictionLow'] = rmstats.MeanDiffListSparse(self.gapscores[:_split],
                                                                            self.gappredictedscores[:_split],
                                                                            self.gapgames[:_split])
            s['MeanDiffScorePrediction'] = rmstats.MeanDiffDict(self.scores, self.predictedscores)
            s['MeanGrade'] = rmstats.mean(self.grades.values())
            s['MeanInitialGrade'] = rmstats.MeanDictList(self.performances, 0)
            s['MedianGrade'] = rmstats.median(self.grades.values())
            s['MedianInitialGrade'] = rmstats.MeanDictList(self.performances, 0)
            s['MinGrade'] = min(self.grades.values())
            s['MinInitialGrade'] = rmstats.MinDictList(self.performances, 0)
            s['StdDevScorePrediction'] = rmstats.StdDevDiffDict(self.scores, self.predictedscores)
            s['StdDevGapScorePrediction'] = rmstats.StdDevDiffListSparse(self.gapscores,
                                                                         self.gappredictedscores,
                                                                         self.gapgames)
            s['StdDevGapScorePredictionHigh'] = rmstats.StdDevDiffListSparse(self.gapscores[-_split:],
                                                                             self.gappredictedscores[-_split:],
                                                                             self.gapgames[-_split:])
            s['StdDevGapScorePredictionLow'] = rmstats.StdDevDiffListSparse(self.gapscores[:_split],
                                                                            self.gappredictedscores[:_split],
                                                                            self.gapgames[:_split])
            s['SumGapPredictedScores'] = sum(self.gappredictedscores)
            s['SumGapResults'] = sum(self.gapgames)
            s['SumGapScores'] = sum(self.gapscores)
            s['SumGrade'] = sum(self.grades.values())
            s['SumGradeWeighted'] = rmstats.SumMultDict(self.games, self.grades)
            s['SumInitialGrade'] = rmstats.SumDictList(self.performances, 0)
            s['SumInitialGradeWeighted'] = rmstats.SumMultListDict(self.games, self.performances, 0)
            s['SumPerfWeighted'] = rmstats.SumMultListDict(self.games, self.performances, -1)
            s['SumPredictedScores'] = sum(self.predictedscores.values())
            s['SumResults'] = sum(self.games.values())
            s['SumScores'] = sum(self.scores.values())

        return deepcopy(s)


    def GetStat(self, stat=None):

        return self.statistics.get(stat, None)
    

    def MaxGrade(self): return self.GetStat('MaxGrade')
    def MaxInitialGrade(self): return self.GetStat('MaxInitialGrade')
    def MeanDiffScorePrediction(self): return self.GetStat('MeanDiffScorePrediction')
    def MeanDiffScorePredictionHigh(self): return self.GetStat('MeanDiffScorePredictionHigh')
    def MeanDiffScorePredictionLow(self): return self.GetStat('MeanDiffScorePredictionLow')
    def MeanGrade(self): return self.GetStat('MeanGrade')
    def MeanInitialGrade(self): return self.GetStat('MeanInitialGrade')
    def MedianGrade(self): return self.GetStat('MedianGrade')
    def MedianInitialGrade(self): return self.GetStat('MedianInitialGrade')
    def MinGrade(self): return self.GetStat('MinGrade')
    def MinInitialGrade(self): return self.GetStat('MinInitialGrade')
    def StdDevScorePrediction(self): return self.GetStat('StdDevScorePrediction')
    def StdDevScorePredictionHigh(self): return self.GetStat('StdDevScorePredictionHigh')
    def StdDevScorePredictionLow(self): return self.GetStat('StdDevScorePredictionLow')
    def SumGapPredictedScores(self): return self.GetStat('SumGapPredictedScores')
    def SumGapResults(self): return self.GetStat('SumGapResults')
    def SumGapScores(self): return self.GetStat('SumGapScores')
    def SumGrade(self): return self.GetStat('SumGrade')
    def SumGradeWeighted(self): return self.GetStat('SumGradeWeighted')
    def SumInitialGrade(self): return self.GetStat('SumInitialGrade')
    def SumInitialGradeWeighted(self): return self.GetStat('SumInitialGradeWeighted')
    def SumPerfWeighted(self): return self.GetStat('SumPerfWeighted')
    def SumPredictedScores(self): return self.GetStat('SumPredictedScores')
    def SumResults(self): return self.GetStat('SumResults')
    def SumScores(self): return self.GetStat('SumScores')


if __name__ == '__main__':

    from os.path import abspath
    from ECFresults import ResultFolder

    f1 = ResultFolder(abspath('results/Results 2005-2006'))
    f2 = ResultFolder(abspath('results/Results 2004-2005'))
    
    a = GradeAnalysis(f1)
    b = GradeAnalysis(f2)
    c = GradeAnalysis(f1)
    d = GradeAnalysis(f1)
    
    a.DoIterations(50)
    b.DoIterations(50)
    c.DoIterations(50,
                   initialgrade = b.grades,
                   finalcalculation = c.GradeDifference40)
    d.DoIterations(50, calculation = d.GradeDifference40)
    
    print a.SumGrade()
    print a.SumGradeWeighted()
    print a.SumPerfWeighted()
    print a.SumInitialGrade()
    print a.SumInitialGradeWeighted()
    print a.MeanGrade()
    print a.MeanInitialGrade()
    print a.MedianGrade()
    print a.MedianInitialGrade()
    print a.MaxGrade()
    print a.MaxInitialGrade()
    print a.MinGrade()
    print a.MinInitialGrade()
    print a.MeanDiffScorePrediction()
    print a.StdDevScorePrediction()
    print

    gstats1 = a.GetGradeStatistics()
    for gs in gstats1:
        print gs, gstats1[gs]
    print

    gstats2 = b.GetGradeStatistics()
    for gs in gstats2:
        print gs, gstats2[gs]
    print

    gstats3 = c.GetGradeStatistics()
    for gs in gstats3:
        print gs, gstats3[gs]
    print

    gstats4 = d.GetGradeStatistics()
    for gs in gstats4:
        print gs, gstats4[gs]
    print
