'''rmstats
'''

import math


def mean(nums):
    '''Return arithmetic mean of list of numbers.
    '''

    return sum(nums) / float(len(nums))


def median(nums):
    '''Return median of a list of numbers.
    '''

    numscopy = nums[:]
    numscopy.sort()
    n = len(numscopy)
    mid = n / 2
    if (n % 2) == 0:
        return (numscopy[mid] + numscopy[mid - 1]) / 2.0
    else:
        return float(numscopy[mid])


def stdev(nums):
    '''Return sample standard deviation of a list of numbers.
    '''

    avg = mean(nums)
    return math.sqrt(sumsq([x - avg for x in nums]) / (len(nums) - 1))


def sumsq(nums):
    '''Return sum of squares of a list of numbers.
    '''

    return sum([x * x for x in nums])


def MaxDictList(a, e):
    '''Return maximum of list of selected element from lists in
    dictionary of lists of numbers.
    '''

    return max([a[x][e] for x in a])


def MedianDictList(a, e):
    '''Return median of list of selected element from lists in
    dictionary of lists of numbers.
    '''

    return median([a[x][e] for x in a])


def MeanDictList(a, e):
    '''Return arithmetic mean of list of selected element from lists in
    dictionary of lists of numbers.
    '''

    return mean([a[x][e] for x in a])


def MeanDiffDict(a, b):
    '''Return arithmetic mean of list of differences between two
    dictionaries of numbers with identical keys.
    '''

    return mean([a[x] - b[x] for x in a])


def MeanDiffDictList(a, b, e):
    '''Return arithmetic mean of list of differences between selected
    element of two dictionaries of lists of numbers with identical keys.
    '''

    return mean([a[x][e] - b[x][e] for x in a])


def MeanDiffList(a, b):
    '''Return arithmetic mean of list of differences between two
    lists of numbers with identical length.
    '''

    return mean([x - y for (x, y) in zip(a, b)])


def MeanDiffListSparse(a, b, c):
    '''Return arithmetic mean of list of differences between two
    lists of numbers with identical length if the number in the
    third list is greater than 0.
    '''

    return mean([x - y for (x, y, z) in zip(a, b, c) if z > 0])


def MinDictList(a, e):
    '''Return minimum of list of selected element from lists in
    dictionary of lists of numbers.
    '''

    return min([a[x][e] for x in a])


def StdDevDiffDict(a, b):
    '''Return standard deviation of list of differences between two
    dictionaries of numbers with identical keys.
    '''

    return stdev([a[x] - b[x] for x in a])


def StdDevDiffListSparse(a, b, c):
    '''Return standard deviation of list of differences between two
    lists of numbers with identical length if the number in the
    third list is greater than 0.
    '''

    return stdev([x - y for (x, y, z) in zip(a, b, c) if z > 0])


def SumDictList(a, e):
    '''Return sum of list of selected element from lists in
    dictionary of lists of numbers.
    '''

    return sum([a[x][e] for x in a])


def SumMultDict(a, b):
    '''Return sum of products of elements of two dictionaries of
    numbers with identical keys.
    '''

    return sum([a[x] * b[x] for x in a])


def SumMultListDict(a, b, e):
    '''Return sum of products of elements of one dictionary of
    numbers with selected element of dictionary of lists of numbers
    with identical keys.
    '''

    return sum([a[x] * b[x][e] for x in a])
